/*
  
  September 2020 
  MTS Code for wireless EEG AD8232 base station
  receives data via bound HC06 on Serial1
  RTC set to UTC
  Using Timezone to control DST
*/

#include <Wire.h>
//#include <Adafruit_MLX90614.h>
#include "DHT.h"        //DHT22
#include <DS1307RTC.h>   // https://github.com/PaulStoffregen/DS1307RTC
#include <Timezone.h>    // https://github.com/JChristensen/Timezone
#include <SPI.h>

#include <SdFat.h> // Use SdFat so can have .xxxx extension
SdFat SD;

File MISCFile;
const int chipSelect = 8;  // Chip select for SD

// US Mountain Time Zone (Denver)
TimeChangeRule MDT = { "MDT", Second, Sun, Mar, 2, -360 };    //Daylight time = UTC - 6 hours
TimeChangeRule MST = { "MST", First, Sun, Nov, 2, -420 };     //Standard time = UTC - 7 hours
Timezone myTZ(MDT, MST);

TimeChangeRule *tcr;        //pointer to the time change rule, use to get TZ abbrev

//EEG variables
const short numb = 32;     // 1/2 number of data points (must be power of 2)
short data[numb];          // data values

// Prototypes
bool GotData();								 // HC06 got Data from EEG 
void WriteSerial(int, int, int, int, int);   // Write to serial port
void WriteSD(int, int, int, int, int);       // Write to microSD card

void setup(){
  Serial.begin(115200);
  Serial1.begin(19200);		// HC06
// while (!Serial); // M4
  setSyncProvider(RTC.get);   // the function to get the time from the RTC
  if(timeStatus()!= timeSet)
      Serial.println("Unable to sync with the RTC");
  else
      Serial.println("RTC has set the system time");

  if (!SD.begin(chipSelect)){
    Serial.println("Card failed, or not present");
    // don't do anything more:
    while (1);
  }
  MISCFile = SD.open("EEG.csv", FILE_WRITE);
  MISCFile.println("Date, Hr:Min:Sec,C0,C1,C2,C3,C4,C5,C6,C7,C8,C9,C10,C11,C12,C13,C14,C15,C16,C17,C18,C19,C20,C21,C22,C23,C24,C25,C26,C27,C28,C29,C30,C31");
  MISCFile.close();

}
void loop(){
	time_t utc = now();
	time_t local = myTZ.toLocal(utc, &tcr); // This is local time with DST 
 
//WriteSerial(month(local), day(local), hour(local), minute(local), second(local));
//WriteSD(month(local), day(local), hour(local), minute(local), second(local));
//delay(1000);
 
	if(GotData()){
		WriteSerial(month(local), day(local), hour(local), minute(local), second(local));
		WriteSD(month(local), day(local), hour(local), minute(local), second(local));
	}
}

bool GotData() {
  bool newData = false;
  int ndx = 0;
  char startMarker = '$';
  char rc;
  
  while (Serial1.available() > 0) {
    rc = Serial1.read();
    if (rc == startMarker) {
      for (ndx =0; ndx <32; ndx++){
        data[ndx] = Serial1.parseInt();
      }
      newData = true;
    }
  }
  return newData;
}


void WriteSerial(int month, int day, int hour, int minute, int second){   // Write to serial port
  for (short i = 0; i < 31; i++) { 
      Serial.print(data[i]);
      Serial.print(",");
  }
  Serial.println(data[31]);
}

void WriteSD(int month, int day, int hour, int minute, int second){       // Write to microSD card
  MISCFile = SD.open("EEG.csv", FILE_WRITE);
//  MISCFile.println("Date, Hour:Minute:Second, C0 - C31");

    MISCFile.print(month); 
    MISCFile.print('/');
    MISCFile.print(day);
    MISCFile.print(',');
    MISCFile.print(hour);
    MISCFile.print(':');
    MISCFile.print(minute);
    MISCFile.print(':');
    MISCFile.print(second);
    MISCFile.print(',');
    for (short i = 0; i < 31; i++) { 
        MISCFile.print(data[i]);
        MISCFile.print(",");
    }
    MISCFile.println(data[31]);

    MISCFile.close();
 }
